#!/usr/bin/perl -w

our $VERSION='1.20130105';

=head1 NAME

unstickxfoil.pl - kill and xfoil.exe processes that get stuck

=head1 SYNOPSIS

	perl unstickxfoil.pl [options]

or

	unstickxfoil.exe [options]

     Options:
       -help	-?	brief help message
       -man		full documentation
       -show		don't kill anything, just show what we *would* kill.
       -sleep		how long to sleep inbetween checks. (default=30s) Note:-
			* samples for 5 seconds, so sleep=1 will have a delay of 6 seconds (5+1)
			* kills *all* stuck xfoil processes, if *any* one was found stuck 6 times in a row
			  (because stupid lamearse windows doesn't tell us *which* xfoil PID was the stuck one...)
			* stuck means "using >22% CPU".  (xfoil uses either 0%, or 99+%, so 22% should work fine, even on a 4-core system)

			** Probably best not to run interactive xfoil sessions at the same time as batch ones, since an un-stuck xfoil will still get killed even if 6 *different* otherones were each momentarily using >22% CPU

=cut

######################################################################

use strict;
use warnings;		# same as -w switch above

use Getopt::Long;	# Commandline argument parsing
use Pod::Usage;		# Inbuilt documentation helper

my %arg;		# We store copies of commandline arguments in here

# Parse options, and print help/manual if requested.
&GetOptions('help|?' => \$arg{'help'},			# breif instructions
	    'man' => \$arg{'man'},			# complete manual
	    'show'=>\$arg{'show'},
	    'sleep|s=i'=>\$arg{'sleep'},
	   ) or &pod2usage(2); 
no warnings;
	   &pod2usage(1) if ($arg{'help'});
	   &pod2usage(-exitstatus => 0, -verbose => 2) if $arg{'man'};
use warnings;


my $killed=0;
my %stuck;
my $run=0;
my $sleep=30; $sleep=$arg{'sleep'} if(defined $arg{'sleep'} );
my $pretends=''; $pretends=' (pretends to)' if(($arg{'show'})||($arg{'debug'}));

print "This program checks every 30 seconds to see if xfoil.exe is running.  If it is, and the same process continues to run for 3 minutes, this program$pretends kills the 'stuck' xfoil.exe process(es).\n\nThis probably needs to run as Administrator for 'taskkill' to work\n\n";

=for code

"(PDH-CSV 4.0)","\\DRIP\Process(xfoil#1)\% Processor Time","\\DRIP\Process(xfoil)\% Processor Time"
"01/05/2013 20:31:57.171","0.000000","99.688939"
Exiting, please wait...                         
The command completed successfully.

=cut

&killstuck() unless($^O =~/Win32/i);	# different way under unix

while(1) {
  my @dat=`typeperf "\\Process(*xfoil*)\\% Processor Time" -sc 1 -si 5`;
  my @tim=split(/,/,$dat[2]); $tim[0]=0;
  my $ok=1; foreach my $i (@tim) {$i=~s/"//g; $ok=0 if($i>22);}
  my(%new,$xc);
  if($ok) { # Nothing stuck
    my %empty;
    %stuck=%empty;
  } else  {
    # tasklist /fi "STATUS eq not responding"  /v
    @dat=`tasklist /fi "Imagename eq xfoil.exe"`;

=for code

Image Name                     PID Session Name        Session#    Mem Usage
========================= ======== ================ =========== ============
xfoil.EXE                     3884 Console                    1     14,900 K

=cut

    foreach(@dat) {
      next unless(/xfoil.exe\s*(\d+)/i);
      my $pid=$1;

	if($stuck{$pid}++>5) {
	  $stuck{$pid}=0;
	  $killed++;
	  print "killing $pid ($killed)\n";
          `taskkill /F /PID $pid` unless(($arg{'show'})||($arg{'debug'}));
	}
	$new{$pid}=$stuck{$pid};

        $xc.=" " . $pid;
    } # dat
    %stuck=%new;	# Discard old un-stuck stuff
  } # ok

  $run++;
  print "$run\tk=$killed\tok=$ok\txc="; foreach my $k (sort keys %new) { print "$k($new{$k}) ";}; print "\n";
  sleep($sleep);
#  sleep(27) unless(($arg{'show'})||($arg{'debug'}));

} # 1




sub killstuck {
  # my $pid = fork;
  # if ($pid == 0) {		# child
    while(1) {
      sleep($sleep);
      my $xp='';
      if($^O =~/Win32/i) {

	my @dat=`wmic process list`;
	my @cols=split(/\s+/,$dat[0]);
	my($i,$idx); foreach $i (0..$#cols) { if($cols[$i] eq 'ProcessId'){$idx=(($#cols)-$i);last}; }
	if(defined $idx) {
	  foreach(@dat) {
	    if(/\sxfoil.exe\s/) {
	      @cols=split(/\s+/,$_);
	      $xp.=" " . $cols[($#cols)-$idx+2];
	    }
	  }
	} # idx
      } else {
        $xp=`pidof xfoil`;	# Linux
      }

      my %new;
      my @pids=split(/\s+/,$xp);
      foreach my $pid (@pids) { next if($pid!~/\d/);
	if($stuck{$pid}++>5) {
	  $stuck{$pid}=0;
	  $killed++;
	  print "killing $pid ($killed)\n";
	  if($^O =~/Win32/i) {
            `taskkill /F /PID $pid`  unless(($arg{'show'})||($arg{'debug'})); # Windows
	  } else {
	    `kill $pid`  unless(($arg{'show'})||($arg{'debug'})); # linux
	  }
	} # stuck
	$new{$pid}=$stuck{$pid};
      } # pids

      %stuck=%new;	# Discard old non-stuck stuff

    $run++;
    print "$run\tk=$killed\txc="; foreach my $k (sort keys %new) { print "$k($new{$k}) ";}; print "\n";


    } # 1
  # } # child
} # killstuck



# old wmic way:-

sub oldway {
while(1) {
  my %new;
  my @dat=`wmic process list`;
  my @cols=split(/\s+/,$dat[0]);
  my($i,$idx,$xc); foreach $i (0..$#cols) { if($cols[$i] eq 'ProcessId'){$idx=(($#cols)-$i);last}; }
  if(defined $idx) {
    foreach(@dat) {
      if(/\sxfoil.exe\s/) {
        @cols=split(/\s+/,$_);
	my $pid=$cols[($#cols)-$idx+2];

	if($stuck{$pid}++>5) {
	  $stuck{$pid}=0;
	  $killed++;
	  print "killing $pid ($killed)\n";
          `taskkill /F /PID $pid`;
	}
	$new{$pid}=$stuck{$pid};

        $xc.=" " . $cols[($#cols)-$idx+2];
      }
    }
    %stuck=%new;	# Discard old un-stuck stuff
  } # idx

  $run++;
  print "$run\tk=$killed\txc="; foreach my $k (sort keys %new) { print "$k($new{$k}) ";}; print "\n";
  sleep(30);
} # 1
} # oldway
